# $Id: openurl.py 4282 2011-04-17 07:15:48Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.9.8/src/openurl.py $
from bzv.yearnormalize import YearNormalize
from meresco.core import Observable
from urllib import urlencode
from re import compile

# FIXME!
# mapping to OpenURL genre = configuratie
# beter om genormaliseerde genre te gebruiken
genreMap = {
    'chapter'                  : 'bookitem',
    'book review'              : 'article',
    'conference paper'         : 'proceeding',
    'conference proceedings'   : 'conference',
    'doctoral thesis'          : 'book',
    'master thesis'            : 'book',
    'thesis'                   : 'book',
    'lecture'                  : 'book' ,
    'working paper'            : 'preprint'
}

infoRe = compile(r'^(?i)info:(\w+)/(\S+)$')
urnRe  = compile(r'^(?i)urn:(\w+):([\w-]+)$')

def xpath(node, path):
    return node.xpath(path, {'mods':'http://www.loc.gov/mods/v3'})

def xpathString(node, path):
    return ''.join(xpath(node, path)[:1])

def getINFO(identifierString):
    m = infoRe.match(identifierString)
    if not m:
        return '', ''
    type, identifier = m.groups()
    return type, identifier

def getURN(identifierString):
    m = urnRe.match(identifierString)
    if not m:
        return '', ''
    type, identifier = m.groups()
    return type.lower(), identifier

def add(aDictionary, key, value):
    if value:
        aDictionary[key] = value

class OpenURL(Observable):
    def __init__(self, baseurl, infra = None):
        Observable.__init__(self)
        self._baseurl = baseurl
        self.infra = infra
        self.serverConfig  = self.infra.configRoot
        self.genreDict = self.serverConfig.getSectionNormalized('genre')

    def createOpenURL(self, lxmlNode, recordId):
        return '%s?%s' % (self._baseurl, self._create(lxmlNode, recordId))

    def _create(self, lxmlNode, recordId):
        data = {}
        genreFull = xpathString(lxmlNode, 'mods:genre/text()')
        genre =genreFull.split('/')[-1]
        openurlGenre = genre
        if self.genreDict.has_key(genreFull): openurlGenre = self.genreDict[genreFull]
        if genreMap.has_key(openurlGenre): openurlGenre = genreMap[openurlGenre]
        add(data, 'genre', openurlGenre)
        add(data, 'aulast', xpathString(lxmlNode, 'mods:name[@type="personal"][mods:role/mods:roleTerm="aut"]/mods:namePart[@type="family"]/text()'))
        add(data, 'aufirst', xpathString(lxmlNode, 'mods:name[@type="personal"][mods:role/mods:roleTerm="aut"]/mods:namePart[@type="given"]/text()').rstrip('.'))
        title = xpathString(lxmlNode, 'mods:titleInfo/mods:title/text()').split("/")[0]
        relatedTitle = xpathString(lxmlNode, 'mods:relatedItem/mods:titleInfo/mods:title/text()').split("/")[0]
        if openurlGenre in ['article', 'proceeding', 'bookitem'] or genre in ['workingPaper'] or (not (openurlGenre or genre) and relatedTitle):
            add(data, 'atitle', title)
            add(data, 'title', relatedTitle)
        else:
            add(data, 'title', title)
        identifiers = xpath(lxmlNode, '//mods:identifier/text()')
        for identifierString in identifiers:
            typeIdentifier, identifier = getURN(identifierString)
            if not (typeIdentifier or identifier):
                typeIdentifier, identifier = getINFO(identifierString)
            if typeIdentifier in ['issn', 'isbn']:
                add(data, typeIdentifier, identifier.replace('-',''))
            elif typeIdentifier in ['doi']:
                add(data, 'id', typeIdentifier + ':' + identifier)
        volume = xpathString(lxmlNode, 'mods:relatedItem/mods:part/mods:detail[@type="volume"]/mods:number/text()')
        add(data, 'volume', volume.split('-')[0])
        issue = xpathString(lxmlNode, 'mods:relatedItem/mods:part/mods:detail[@type="issue"]/mods:number/text()')
        add(data, 'issue', issue.split('-')[0])
        spage = xpathString(lxmlNode, 'mods:relatedItem/mods:part/mods:extent[@unit="page"]/mods:start/text()')
        epage = xpathString(lxmlNode, 'mods:relatedItem/mods:part/mods:extent[@unit="page"]/mods:end/text()')
        add(data, 'spage', spage)
        add(data, 'epage', epage)
        if spage and epage:
            add(data, 'pages', '%s-%s' % (spage, epage))
        # FIXME: locator weigert epn te gebruiken als date bestaat
        if genre not in ['series', 'journal']:
            add(data, 'date',  YearNormalize(yearRange=(1000,2100)).normalize(xpathString(lxmlNode, 'mods:originInfo/mods:dateIssued/text()')))
            #FIXME in het geval booktitems werkt locator niet goed als er een date is
            #daarom wordt year doorgegeven speciaal voor bzv.js
            add(data, 'year',  YearNormalize(yearRange=(1000,2100)).normalize(xpathString(lxmlNode, 'mods:relatedItem/mods:originInfo/mods:dateIssued/text()')))
        add(data, 'sid', self.any.getSid(recordId))
        add(data, 'epn', xpathString(lxmlNode, 'mods:location/mods:holdingSimple/mods:copyInformation/mods:note[@type="epn"]/text()'))

        #'auinit'
        #'auinit1'
        #'auinitm'
        #'eissn'
        #'coden'
        #'sici'
        #'bici'
        #'part'

        #'artnum'
        #'date'
        #'ssn'
        #'quarter'

        return urlencode(data)
