# -*- coding: utf-8 -*-
# $Id: storagecomponent.py 3959 2010-09-14 07:16:09Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.9.8/src/storagecomponent.py $

from storage import HierarchicalStorage, Storage
from cStringIO import StringIO
from bzv.boushistorage import BoushiStorage
from meresco.components.storagecomponent import StorageComponent, defaultSplit, defaultJoin
from lxml.etree import parse, tostring, XSLT

class BZVStorageComponent(StorageComponent):
    def __init__(self, directory, split=defaultSplit, join=defaultJoin, revisionControl=False, partsRemovedOnDelete=[], nsMap=[], xslt=[], storageType='boushi'):
        self._partsRemovedOnDelete = partsRemovedOnDelete
        self._nsMap = nsMap
        for s in xslt:
            if   "picaplus2mods" in s: self._picaplusXslt = XSLT(parse(open(s)))
            elif "metadata2mods" in s: self._metadataXslt = XSLT(parse(open(s)))
        if storageType == 'filesystem':
            self._storage = HierarchicalStorage(Storage(directory, revisionControl=revisionControl, ), split, join)
        elif storageType == 'boushi':
            if revisionControl:
                raise NotImplementedError("boushi storage type does not support revision control")
            self._storage = BoushiStorage(directory, split, join)
        else:
            raise NotImplementedError("unknown storage type '%s'" % str(storageType))

    def isPartAvailable(self, id, partName):
        return (id, partName) in self._storage

    def getStream(self, id, partName):
        if partName == 'mods':
            return self._getMods(id)
        return self._storage.get((id, partName))

    def yieldRecord(self, id, partName):
        stream = self.getStream(id, partName)
        for data in stream:
            yield data
        stream.close()

    def deleteDocument(self, id):
        for x, part in self._storage.glob((id, None)):
            self._storage.deletePart(id, part)

    def write(self, sink, id, partName):
        stream = self._storage.get((id, partName))
        try:
            for line in stream:
                sink.write(line)
        finally:
            stream.close()

    def _listIdentifiersByPartName(self, partName, identifierPrefix=''):
        # overriding private method in super class
        # this method is used in listIdentifiers()
        for identifier, partname in self.glob((identifierPrefix, partName)):
            if partName == "deleted":
                yield identifier
            elif self.isAvailable(identifier, "deleted") == (True, False):
                yield identifier
                            
    def _getMods(self, id):
        metadata = parse(self._storage.get((id, "metadata")))
        mods = metadata.xpath("//mods:mods", namespaces=self._nsMap)
        if mods:
            mods = self._metadataXslt(metadata)
        else:
             mods = self._picaplusXslt(metadata)
        return StringIO(tostring(mods))
